from typing import List, Optional

from vk_urls_validator.utils import exceptions
import string

ALLOWED_CHARACTERS = string.digits + string.ascii_uppercase + string.ascii_lowercase + '.'


def get_hostname(hostname: str, hostnames: List[str]) -> Optional[str]:
    for prefix in hostnames:
        if hostname.startswith(prefix):
            return prefix
    return None


def validate_hash(url: str, url_hash: str) -> Optional[str]:
    """
    Hash validating by VK rules (https://vk.com/faq18038)

    :param url: URL
    :param url_hash: URL hash
    :return:
    """

    url_hash = url_hash.strip('/').strip()

    if url_hash.startswith('id') and url_hash[2:].isnumeric():
        return url_hash

    if url_hash.startswith('public') and url_hash[6:].isnumeric():
        return url_hash

    if url_hash.startswith('club') and url_hash[5:].isnumeric():
        return url_hash

    if not 5 <= len(url_hash) <= 32:
        raise exceptions.VKInvalidHashError(url=url)

    if url_hash[:3].isnumeric():
        raise exceptions.VKInvalidHashError(url=url)

    if url_hash.startswith('_') or url_hash.endswith('_'):
        raise exceptions.VKInvalidHashError(url=url)

    for i in range(len(url_hash)):
        if url_hash[i] not in ALLOWED_CHARACTERS:
            raise exceptions.VKInvalidHashError(url=url)

        if url_hash[i] == '.':
            if len(url_hash[i:]) < 4:
                raise exceptions.VKInvalidHashError(url=url)

            if not (url_hash[i + 1].isalpha() and url_hash[i + 2].isalpha() and url_hash[i + 3].isalpha() and url_hash[
                i + 4].isalpha()):
                raise exceptions.VKInvalidHashError(url=url)

    characters = set(url_hash)
    if characters.discard(set(ALLOWED_CHARACTERS)):
        raise exceptions.VKInvalidHashError(url=url)

    return url_hash



from urllib.parse import urlparse

from vk_urls_validator.utils import exceptions, utils

MAIN_HOST = 'vk.com'
POSSIBLE_HOSTS = ['vk.com', 'm.vk.com', 'vk.ru']


def validate_url(url: str) -> str:
    """
    Validate VK URL

    Parameters:
        url (``str``):
            Url to validate

    Returns:
        :obj:`str`: validated URL

    Raises:
        CantFindHostnameError:
            Can't find URL's hostname.

        HostNotInPossibleHostsError:
            URL's host not in possible hosts.

        VKInvalidHashError:
            URL's hash does not comply with VK rules.
    """
    if url.startswith('https://') or url.startswith('http://'):
        parsed_url = urlparse(url=url)
    else:
        parsed_url = urlparse(url=f'https://{url}')

    if parsed_url.hostname is None:
        raise exceptions.CantFindHostnameError(url=url)

    hostname = utils.get_hostname(hostname=parsed_url.hostname, hostnames=POSSIBLE_HOSTS)
    if hostname is None:
        raise exceptions.HostNotInPossibleHostsError(url=url, possible_hosts=POSSIBLE_HOSTS)

    url_hash = utils.validate_hash(url=url, url_hash=parsed_url.path)

    return f'https://vk.com/{url_hash}'



from typing import List


class BaseValidationError(Exception):
    pass


class HostNotInPossibleHostsError(BaseValidationError):
    def __init__(self, url: str, possible_hosts: List[str]):
        self.url = url
        self.possible_hosts = possible_hosts

    def __str__(self):
        return f"URL's ({self.url}) host not in possible hosts: {', '.join(self._get_possible_hosts_in_quotes())}"

    def _get_possible_hosts_in_quotes(self) -> List[str]:
        return [f'"{prefix}"' for prefix in self.possible_hosts]


class CantFindHostnameError(BaseValidationError):
    def __init__(self, url: str):
        self.url = url

    def __str__(self):
        return f"Can't find hostname for URL {self.url}"


class BaseVKValidationError(BaseValidationError):
    pass


class VKInvalidHashError(BaseVKValidationError):
    def __init__(self, url: str):
        self.url = url

    def __str__(self):
        return f'URL {self.url} has invalid hash'



import os

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

with open("README.md", encoding="utf-8") as f:
    readme = f.read()

VERSION = '0.0.1'
DESCRIPTION = 'Validator for VK URLs.'

# Setting up
setup(
    name="vk_urls_validator",
    version=VERSION,
    author="Emir Takhaviev",
    author_email="tah116emir@outlook.com",
    description=DESCRIPTION,
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/Wiped-Out/vk_urls_validator",
    download_url="https://github.com/Wiped-Out/vk_urls_validator/releases/latest",
    packages=find_packages(exclude=["tests*"]),
    keywords=['python', 'vk urls', 'validation'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)
